"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.VanillaEventBus = void 0;
const tslib_1 = require("tslib");
const EventBusValidator_1 = require("./EventBusValidator");
const eventbus_utils_1 = require("./eventbus-utils");
class VanillaEventBus {
    constructor(config) {
        this.config = config;
        this.subscribeTopics = [];
        this.publishTopics = {};
        this.init = () => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (this.config.publisher && this.config.publisher.publishInit) {
                const topicnames = (0, eventbus_utils_1.getRootQueryNames)(this.config.publisher.schema);
                yield this.config.publisher.publishInit(topicnames);
            }
            if (this.config.subscriber) {
                yield this.consume(this.config.subscriber);
            }
            // this.ping();
            // setInterval(() => this.ping(), 15 * 1000);
        });
        this.consume = (args) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (!this.consumeValidator) {
                throw new Error("Subcriber config not added");
            }
            const cb = args.cb(this);
            yield this.consumeValidator
                .validateConsumerQueries(args.queries)
                .then((topics) => {
                topics.forEach((topicName) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
                    yield args.getSubscription(topicName, (data) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
                        var _a, _b, _c, _d;
                        (_a = this.config.plugins) === null || _a === void 0 ? void 0 : _a.forEach((plugin) => {
                            plugin.consumeStart &&
                                plugin.consumeStart({ topic: topicName });
                        });
                        try {
                            const eventData = yield ((_b = this.consumeValidator) === null || _b === void 0 ? void 0 : _b.extractData(topicName, data));
                            yield cb({ key: topicName, data: eventData });
                        }
                        catch (e) {
                            (_c = this.config.plugins) === null || _c === void 0 ? void 0 : _c.forEach((plugin) => {
                                plugin.consumeError &&
                                    plugin.consumeError(e);
                            });
                        }
                        (_d = this.config.plugins) === null || _d === void 0 ? void 0 : _d.forEach((plugin) => {
                            plugin.consumeEnd &&
                                plugin.consumeEnd({ topic: topicName });
                        });
                    }));
                }));
            });
        });
        this.publish = (props) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            var _a, _b, _c, _d;
            if (!this.publishValidator) {
                throw new Error("Publish config not added!");
            }
            (_a = this.config.plugins) === null || _a === void 0 ? void 0 : _a.forEach((plugin) => {
                plugin.publishStart &&
                    plugin.publishStart({ topic: props.topicName });
            });
            try {
                yield this.publishValidator.publishValidate(props);
                (_b = this.config.plugins) === null || _b === void 0 ? void 0 : _b.forEach((plugin) => {
                    plugin.publishEnd &&
                        plugin.publishEnd({ topic: props.topicName });
                });
            }
            catch (e) {
                (_c = this.config.plugins) === null || _c === void 0 ? void 0 : _c.forEach((plugin) => {
                    plugin.publishError && plugin.publishError(e);
                });
                (_d = this.config.plugins) === null || _d === void 0 ? void 0 : _d.forEach((plugin) => {
                    plugin.publishEnd &&
                        plugin.publishEnd({ topic: props.topicName });
                });
                throw e;
            }
            return;
        });
        this.closeConsumer = () => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () { });
        this.closePublisher = () => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (this.config.publisher && this.config.publisher.publishClose) {
                const topicnames = (0, eventbus_utils_1.getRootQueryNames)(this.config.publisher.schema);
                yield this.config.publisher.publishClose(topicnames);
            }
        });
        if (config.publisher) {
            this.publishValidator = new EventBusValidator_1.EventBusValidator({
                publisherSchema: config.publisher.schema,
            });
        }
        if (config.subscriber) {
            this.consumeValidator = new EventBusValidator_1.EventBusValidator({
                publisherSchema: config.subscriber.schema,
            });
        }
    }
}
exports.VanillaEventBus = VanillaEventBus;
// (async () => {
//   const bus = new PubSubEventBus({
//     serviceName: "",
//   });
//   setInterval(() => {
//     console.log(bus.healthCheck());
//   }, 1000);
//   await bus.init();
// })();
